function result = MUL(op1,op2,fxd, DBG,dbgStr)
%MUL	simulate the behavior of the VHDL description of a MULtiplier
%	result = MUL(op1,op2,fxd) returns the (signed) result of the 
%	multiplication of 'op1' and 'op2' (both signed). 
%	The binary fixed-point format of 'result', as well as that of 'op1' 
%	and 'op2' should be given in 'fxd' as [N M], where N denotes the 
%	total number of bits, while M defines the number of fractional bits.
%
%	See also ALU. 

% (c) H.J. Lincklaen Arriens, 
%     Delft University of Technology, November 2004


N = fxd(1);
M = fxd(2);
if ( N <= M )
	error( 'Incorrect values for parameter Fixed ...' );
end

if ~exist('dbgStr','var')
	dbgStr = '';
end

maxPos  =  2^(N-1) -1;
maxNeg  = -2^(N-1);
fracFac =  2^M;

op1Int = op1 * fracFac;
if ( floor(op1Int) ~= op1Int )
	warning( 'MUL: op1 doesn''t fit in %d fractional bits ...', M );
	op1Int = floor(op1Int);
	warning( 'MUL: op1 truncated to %15.10f ...', op1Int/fracFac );
end 
if ((op1Int > maxPos) || (op1Int < maxNeg))
	error( 'MUL ERROR: signed op1 doesn''t fit in %d integer bits ...', N-M );
end

op2Int = op2 * fracFac;
if ( floor(op2Int) ~= op2Int )
	warning( 'MUL: op2 doesn''t fit in %d fractional bits ...', M );
	op2Int = floor(op2Int);
	warning( 'MUL: signed op2 truncated to %15.10f ...', op2Int/fracFac );
end 
if ((op2Int > maxPos) || (op2Int < maxNeg))
	error( 'MUL ERROR: signed op2 doesn''t fit in %d integer bits ...', N-M );
end

temp   = op1Int * op2Int;
% now truncate to exactly fit in the correct number of fractional bits
result = floor( temp / fracFac ) / fracFac;

if DBG
	fprintf( '%s   %s = %s * %s\n', dbgStr, fixp2hex(result,fxd), ...
	                           			fixp2hex(op1,fxd), fixp2hex(op2,fxd) ); 
end
