function tFrames = alap(adjMat,delayVec)
%ALAP	Find the SSG using the ALAP	method
%	tFrames = ALAP(adjMat,delayVec) returns a two-column vector 'tFrames'
%	describing the SSG (Scheduled Sequencing Graph) of the operations 
%	when scheduled according to the ALAP (As Late As Possible) method
%	(no resource constraints).
%	The unscheduled flow has to be given in	'adjMat' with the clock delays 
%	per resource type (e.g. MUL, ALU) as integer number of clock cycles 
%	specified in 'delayVec'.
%
%	See also ASAP, FORCED, LISTSCHED, TOADJMAT, PARSE. 

% (c) H.J. Lincklaen Arriens, 
%     Delft University of Technology, November 2004


% use asap routine with an up-down flipped timing model
T = asap(adjMat',delayVec);
% correct for flipped timing model
singleT       = ( T(:,2) == 0 );
tMax          = max(max(T));
T             = tMax(ones(size(T))) - T +1;
T(~singleT,:) = fliplr( T(~singleT,:) );
T(singleT,2)  = 0;
tFrames       = T;
